//***************************************************************************
//
//  File........: main.c
//
//  Author(s)...: ATMEL Norway
//
//  Target(s)...: ATmega169
//
//  Compiler....: AVR-GCC 3.3.1; avr-libc 1.0
//
//  Description.: AVR Butterfly main module
//
//  Revisions...: 1.0
//
//  YYYYMMDD - VER. - COMMENT                                       - SIGN.
//
//  20030116 - 1.0  - Created                                       - KS
//  20031009          port to avr-gcc/avr-libc                      - M.Thomas (*)
//  20031204          fixed imcompatibility with sleep-modes        - mt
//  20040218          fixed 'logical and' in calibration            - shc/mt
//  20050827          fixed avr-libc iom169.h compatibility
//                    added keyklick function (from version6)       - mt/v6
//  20051017          DSLR-Timer based on AVR Butterfly eval-code	- Michael P.
//
//***************************************************************************

// (*) Martin Thomas, Kaiserslautern, Germany, e-mail: mthomas(at)rhrk.uni-kl.de 
// or eversmith(at)heizung-thomas.de
//
// I'm not working for ATMEL.
// The port is based on REV_06 of the ATMEL-Code (for IAR-C)
// Initialy I marked my changes with "// mt" or enclosed them with "// mtA" and 
// "// mtE" but forgot this for some changes esp. during debugging. 'diff' 
// against the original code to see everything that has been changed.

// 17. october 2005
// Michael Posavec, Vienna, Austria, e-mail: lateralusmail(at)yahoo.de
// This code is based on the gcc port from Martin Thomas, and I'm not working for
// ATMEL either.
// I didn't want to spend any money buying an expensive timer-remote-control for my
// DSLR, thus I searched for a development board with sepecific features
// (available module including LCD, some IO's, some buttons, free development tools,
// no additional hardware eg.: JTAG-box needed for flashing of code) and found the
// perfect solution in ATMEL's AVR butterfly demo board. Using the code below in
// conjunction with the AVR butterfly you just have to upload the code via RS232
// to the board and you can use PB2 on J400 as shutter-signal for your Canon EOS-DSLR.
// (GND is also available on J400)



//mtA
//#include <inavr.h>
//#include "iom169.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <inttypes.h>
//mtE

#include "main.h"
#include "LCD_functions.h"
#include "LCD_driver.h"
#include "button.h"
#include "RTC.h"
#include "timer0.h"
#include "BCD.h"
#include "usart.h"
#include "sound.h"
#include "ADC.h"
#include "dataflash.h"
#include "eeprom169.h"

//#include "eeprom.h"
// mt Test() is not realy needed 
//- can not be accessed without external hardware
#include "test.h"	
#include "vcard.h"
//#include "menu.h"

#define pLCDREG_test (*(char *)(0xEC))


#define AF_ON			(PORTB &= ~0x01)
#define AF_OFF			(PORTB |= 0x01)

#define SHUTTER_ON		(PORTB &= ~0x04)
#define SHUTTER_OFF		(PORTB |= 0x04)

#define LED_ON			(PORTB &= ~0x08)
#define LED_OFF			(PORTB |= 0x08)

#define LED_ON_TIME		3

//mt: extern __flash unsigned int LCD_character_table[];
//    but this is not used here anyway...
extern unsigned int LCD_character_table[] PROGMEM;

extern volatile uint16_t gPowerSaveTimer;    	// external Counter from "RTC.c"
#define  POWER_SAVE_TIMEOUT 30     				// Initial value, enable power save mode after 30 min

BOOL KeyClickStatus = FALSE;    // Variable to enable/disable keyclick

char gAutoPressJoystick = FALSE;// global variable used in "LCD_driver.c"


unsigned char state;            // helds the current state, according to 
                                // "menu.h"
int  gShootMin, gShootSec, gWaitMin, gWaitSec, gShootCount;
int  gShootCountTemp, gLED, gLEDtimer;
char gClick,gBuzzer;


/*****************************************************************************
*
*   Function name : main
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Contains the main loop of the program
*
*****************************************************************************/
// mt __C_task void main(void)
int main(void)
{    
//    unsigned char state, nextstate;
    char input;
	int ADC_value;
	unsigned char minH,minL,secH,secL;


    // Program initalization
    Initialization();
    sei(); // mt __enable_interrupt();

	//KeyClickStatus=TRUE;
	//ADMUX = TEMPERATURE_SENSOR;
	ADMUX = LIGHT_SENSOR;
	
	ADC_init(LIGHT_SENSOR);
    ADC_value = ADC_read();         // Read the light value
	
	state=ST_DEFAULT;
	gShootSec=eeprom_read_byte_169(E2PROM_SHOOTSEC);	// read out the shooting time seconds from EEPROM
	// empty E2PROM has every single bit set
	if (gShootSec == 0xFF)	// if this is the very first time, reset the value to 0
		gShootSec =0;

	gShootMin=eeprom_read_byte_169(E2PROM_SHOOTMIN);	// read out the shooting time minutes from EEPROM
	// empty E2PROM has every single bit set
	if (gShootMin == 0xFF)	// if this is the very first time, reset the value to 0
		gShootMin =0;

	gWaitSec=eeprom_read_byte_169(E2PROM_WAITSEC);	// read out the shooting time seconds from EEPROM
	// empty E2PROM has every single bit set
	if (gWaitSec == 0xFF)	// if this is the very first time, reset the value to 0
		gWaitSec =0;

	gWaitMin=eeprom_read_byte_169(E2PROM_WAITMIN);	// read out the shooting time minutes from EEPROM
	// empty E2PROM has every single bit set
	if (gWaitMin == 0xFF)	// if this is the very first time, reset the value to 0
		gWaitMin =0;

	gShootCount=eeprom_read_byte_169(E2PROM_SHOOTCOUNT);	// read out the shooting counter from EEPROM
	// empty E2PROM has every single bit set
	if (gShootCount == 0xFF)	// if this is the very first time, reset the value to 0
		gShootCount =0;

	gClick=eeprom_read_byte_169(E2PROM_CLICK);	// read out the click setting from EEPROM
	// empty E2PROM has every single bit set
	if (gClick == 0xFF)	// if this is the very first time, reset the value to 0
		gClick =0;
	KeyClickStatus= gClick;		// update the status

	gLED=eeprom_read_byte_169(E2PROM_LED);	// read out the LED setting from EEPROM
	// empty E2PROM has every single bit set
	if (gLED == 0xFF)	// if this is the very first time, reset the value to 0
		gLED =LED_STATE_OFF;

	gBuzzer=eeprom_read_byte_169(E2PROM_BUZZER);	// read out the Buzzer setting from EEPROM
	// empty E2PROM has every single bit set
	if (gBuzzer == 0xFF)	// if this is the very first time, reset the value to 0
		gBuzzer =0;

	Sound_Init();

	while(1)
	{	
		input = getkey();           // Read buttons
		if (gPowerSaveTimer>=POWER_SAVE_TIMEOUT)
		{
			// return to bootloader in power save mode -> this draws only 8,5 uA from the battery
    	    WDTCR = (1<<WDCE) | (1<<WDE);     //Enable Watchdog Timer to give reset
   	    	while(1);   	// wait for watchdog-reset, since the BOOTRST-fuse is 
		                    	// programmed, the Boot-section will be entered upon reset
		}

		// if LED is turned off all the time
		if (gLED == LED_STATE_OFF)
			LED_OFF;

		// if LED is turned on all the time
		if (gLED == LED_STATE_ON)
			LED_ON;

		// if LED is turned on only at night, and turned off during the day
		if (gLED == LED_STATE_LIGHT)
		{
			ADMUX = LIGHT_SENSOR;
			ADC_init(LIGHT_SENSOR);
		    ADC_value = ADC_read();         // Read the light value

			if (ADC_value > 0x3E8)
				LED_ON;						// turn LED on during night
			if (ADC_value < 0x3D8)
				LED_OFF;					// turn LED off during day
		}
		
		// if LED is turned for LED_ON_TIME seconds after any keypress event and
		// LED_ON_TIME seconds before expiration of the shooting- or wait-time
		if (gLED == LED_STATE_AUTO)
		{
			if (input != KEY_NULL)
				gLEDtimer=LED_ON_TIME;		// start the LED timer if keypress event

			if (gLEDtimer)				
				LED_ON;						// turn on LED if timer is running
			else
				LED_OFF;					// turn off ohterwise
		}

/************* DEFAULT STATE *****************/
		if (state == ST_DEFAULT)
		{
			// the default state with its "TIMER" LCD-output is only displayed once during system boot
			// this state is never entered again
		    LCD_puts_f(PSTR("TIMER"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_OPTION;
			if (input == KEY_MINUS)
				state = ST_AUTO;
		}
/************* AUTO STATE ********************/
		else if (state == ST_AUTO)
		{
		    LCD_puts_f(PSTR("AUTO"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_OPTION;		// skip over default state since this is only
										// the welcome screen "TIMER"
			if (input == KEY_MINUS)
				state = ST_MAN;

			if (input == KEY_NEXT)
				state = ST_AUTO_START;
		}
/************* MANUAL STATE ******************/
		else if (state == ST_MAN)
		{
		    LCD_puts_f(PSTR("MAN"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_AUTO;
			if (input == KEY_MINUS)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_MAN_STOP;			
		}
/************* SET STATE *********************/
		else if (state == ST_SET)
		{
		    LCD_puts_f(PSTR("SET"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_MAN;
			if (input == KEY_MINUS)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_SHOOT_TIME;
		}
/************* OPTION STATE ******************/
		else if (state == ST_OPTION)
		{
		    LCD_puts_f(PSTR("OPTION"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_SET;
			if (input == KEY_MINUS)		// skip over default state since this is only
				state = ST_AUTO;		// the welcome screen "TIMER"

			if (input == KEY_NEXT)
				state = ST_OPTION_BUZZER;
		
			// enter bootloader if ENTER is pressed while in OPTION menu
			if (input == KEY_ENTER)
			{
	    	    WDTCR = (1<<WDCE) | (1<<WDE);     //Enable Watchdog Timer to give reset
    	    	while(1);   	// wait for watchdog-reset, since the BOOTRST-fuse is 
		                    	// programmed, the Boot-section will be entered upon reset
			}
		}

/* * * * * * * * S U B M E N U s * * * * * * */
/************* MAN STOP **********************/
		else if (state == ST_MAN_STOP)
		{
		    LCD_puts_f(PSTR("MSTOP"),0);	// output string to LCD
			SHUTTER_OFF;					// deactivate shutter	

			if (input == KEY_PREV)
				state = ST_MAN;

			if (input == KEY_ENTER)
				state = ST_MAN_START;
		}
/************* MAN START *********************/
		else if (state == ST_MAN_START)
		{
		    LCD_puts_f(PSTR("MSTART"),0);	// output string to LCD
			SHUTTER_ON;						// activate shutter	

			if (input == KEY_PREV)
			{
				SHUTTER_OFF;				// deactivate shutter	
				state = ST_MAN;
			}

			if (input == KEY_ENTER)
				state = ST_MAN_STOP;
		}

/************* AUTO START ********************/
		else if (state == ST_AUTO_START)
		{
		    LCD_puts_f(PSTR("ASTART"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_AUTO;
			
			if (input == KEY_ENTER)
			{
				state = ST_AUTO_RUN_SHOOT;
				gShootCountTemp = gShootCount;		// get the number of cycles
				gSECOND=gShootSec + gShootMin*60;	// calculate shoot time in seconds
				gSECOND++;							// add one second, because camera does not start immediately
				Timer0_Init();						// reset the 1s-timer
			}
		}
/************* AUTO RUN SHOOT ****************/
		else if (state == ST_AUTO_RUN_SHOOT)
		{
			SHUTTER_ON;			// activate shutter	

			LCD_Colon(TRUE);	// turn on the ":" on LCD

		    gPowerSaveTimer = 0;                // Reset the Auto Power Down timer

			// if only LED_ON_TIME seconds are left
			if (gSECOND == LED_ON_TIME)
			{
				gLEDtimer = LED_ON_TIME;				// set the LED timer to turn on the LED for
														// the last LED_ON_TIME seconds.
				// if Buzzer is enabled
				if (gBuzzer)
					TCCR1A = 0xC1;		// turn on fast PWM mode
			}

			// turn off the buzzer after 1s
			if (gSECOND == (LED_ON_TIME-1))
				TCCR1A = 0x00;		// turn off fast PWM mode

			// if shooting time has expired
			if (gSECOND <= 0)
			{
				gShootCountTemp--;						// decrease the number of cycles to come

				// if all the cycles have been executed -> exit auto mode
				if (gShootCountTemp == 0)
				{
					SHUTTER_OFF;						// deactivate shutter	
					state = ST_AUTO_START;				// return to AUTO START state
					LCD_Colon(FALSE);					// turn off the ":" on LCD
				}
				else
				{
					state = ST_AUTO_RUN_WAIT;			// next state is the wait time
					gSECOND=gWaitSec + gWaitMin*60;		// calculate wait time in seconds
				}
			}


			minH = gSECOND / 60;
			secH = gSECOND % 60;

			minH = CHAR2BCD2(minH);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			secH = CHAR2BCD2(secH);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';
				
            LCD_putc(0, 'S');
   	        LCD_putc(1, ' ');
       	    LCD_putc(2, minH);
           	LCD_putc(3, minL);
            LCD_putc(4, secH);
   	        LCD_putc(5, secL);
       	    LCD_putc(6, '\0');

			// exit at anytime with PREV key
			if (input == KEY_PREV)
			{
					SHUTTER_OFF;						// deactivate shutter	
					state = ST_AUTO_START;				// return to AUTO START state
					LCD_Colon(FALSE);					// turn off the ":" on LCD
			}

		}
/************* AUTO RUN WAIT *****************/
		else if (state == ST_AUTO_RUN_WAIT)
		{
			SHUTTER_OFF;			// deactivate shutter	

		    gPowerSaveTimer = 0;                // Reset the Auto Power Down timer

			// if shooting time has expired
			if (gSECOND <= 0)
			{
				state = ST_AUTO_RUN_SHOOT;			// next state is again the shoot time
				gSECOND=gShootSec + gShootMin*60;	// calculate shoot time in seconds
				gSECOND++;							// add one second, because camera does not start immediately
				Timer0_Init();						// reset the 1s-timer
			}

			// if only LED_ON_TIME seconds are left
			if (gSECOND == LED_ON_TIME)
				gLEDtimer = LED_ON_TIME;				// set the LED timer to turn on the LED for

			minH = gSECOND / 60;
			secH = gSECOND % 60;

			minH = CHAR2BCD2(minH);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			secH = CHAR2BCD2(secH);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';
				
            LCD_putc(0, 'W');
   	        LCD_putc(1, ' ');
       	    LCD_putc(2, minH);
           	LCD_putc(3, minL);
            LCD_putc(4, secH);
   	        LCD_putc(5, secL);
       	    LCD_putc(6, '\0');

			// exit at anytime with PREV key
			if (input == KEY_PREV)
			{
					SHUTTER_OFF;						// deactivate shutter	
					state = ST_AUTO_START;				// return to AUTO START state
					LCD_Colon(FALSE);					// turn off the ":" on LCD
			}
		}
/************* SHOOT TIME ********************/
		else if (state == ST_SHOOT_TIME)
		{
		    LCD_puts_f(PSTR("SHOOT"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_SHOOT_COUNT;
			if (input == KEY_MINUS)
				state = ST_WAIT_TIME;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_SHOOT_TIME_SET_SEC;
			
		}
/************* SHOOT TIME SET SEC ************/
		else if (state == ST_SHOOT_TIME_SET_SEC)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD
			gArrow10=TRUE;		// turn on the arrow under der seconds-digits

			if (input == KEY_PLUS)
				gShootSec++;
			if (input == KEY_MINUS)
				gShootSec--;

			if (gShootSec > 59)
				gShootSec = 59;

			if (gShootSec < 0)
				gShootSec = 0;

			if ( (input == KEY_PREV) || (input == KEY_NEXT) )
			{
				state = ST_SHOOT_TIME_SET_MIN;
				gArrow10=FALSE;		// turn off the arrow under der seconds-digits
				eeprom_write_byte_169(E2PROM_SHOOTSEC,gShootSec);	// store the changed setting in EEPROM
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				gArrow10=FALSE;		// turn off the arrow under der seconds-digits
				eeprom_write_byte_169(E2PROM_SHOOTSEC,gShootSec);	// store the changed setting in EEPROM
			}

			minH = CHAR2BCD2(gShootMin);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			
			secH = CHAR2BCD2(gShootSec);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';

            LCD_putc(0, 'S');
            LCD_putc(1, ' ');
            LCD_putc(2, minH);
            LCD_putc(3, minL);
            LCD_putc(4, secH);
            LCD_putc(5, secL);
            LCD_putc(6, '\0');
		}
/************* SHOOT TIME SET MIN ************/
		else if (state == ST_SHOOT_TIME_SET_MIN)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD
			gArrow8=TRUE;		// turn on the arrow under der minutes-digits

			if (input == KEY_PLUS)
				gShootMin++;
			if (input == KEY_MINUS)
				gShootMin--;

			if (gShootMin > 99)
				gShootMin = 99;

			if (gShootMin < 0)
				gShootMin = 0;

			if ( (input == KEY_PREV) || (input == KEY_NEXT) )
			{
				state = ST_SHOOT_TIME_SET_SEC;
				gArrow8=FALSE;		// turn off the arrow under der minutes-digits
				eeprom_write_byte_169(E2PROM_SHOOTMIN,gShootMin);	// store the changed setting in EEPROM
			}

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				gArrow8=FALSE;		// turn off the arrow under der seconds-digits
				eeprom_write_byte_169(E2PROM_SHOOTMIN,gShootMin);	// store the changed setting in EEPROM
			}

			minH = CHAR2BCD2(gShootMin);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			
			secH = CHAR2BCD2(gShootSec);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';

            LCD_putc(0, 'S');
            LCD_putc(1, ' ');
            LCD_putc(2, minH);
            LCD_putc(3, minL);
            LCD_putc(4, secH);
            LCD_putc(5, secL);
            LCD_putc(6, '\0');
		}
/************* WAIT TIME *********************/
		else if (state == ST_WAIT_TIME)
		{
		    LCD_puts_f(PSTR("WAIT"),0);		// output string to LCD

			if (input == KEY_PLUS)
				state = ST_SHOOT_TIME;
			if (input == KEY_MINUS)
				state = ST_SHOOT_COUNT;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_WAIT_TIME_SET_SEC;
		}
/************* WAIT  TIME SET SEC ************/
		else if (state == ST_WAIT_TIME_SET_SEC)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD
			gArrow10=TRUE;		// turn on the arrow under der seconds-digits

			if (input == KEY_PLUS)
				gWaitSec++;
			if (input == KEY_MINUS)
				gWaitSec--;

			if (gWaitSec > 59)
				gWaitSec = 59;

			if (gWaitSec < 0)
				gWaitSec = 0;

			if ( (input == KEY_PREV) || (input == KEY_NEXT) )
			{
				state = ST_WAIT_TIME_SET_MIN;
				gArrow10=FALSE;		// turn off the arrow under der seconds-digits
				eeprom_write_byte_169(E2PROM_WAITSEC,gWaitSec);	// store the changed setting in EEPROM
			}

			if (input == KEY_ENTER)
			{
				state = ST_WAIT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				gArrow10=FALSE;		// turn off the arrow under der seconds-digits
				eeprom_write_byte_169(E2PROM_WAITSEC,gWaitSec);	// store the changed setting in EEPROM
			}

			minH = CHAR2BCD2(gWaitMin);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			
			secH = CHAR2BCD2(gWaitSec);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';

            LCD_putc(0, 'W');
            LCD_putc(1, ' ');
            LCD_putc(2, minH);
            LCD_putc(3, minL);
            LCD_putc(4, secH);
            LCD_putc(5, secL);
            LCD_putc(6, '\0');
		}
/************* WAIT  TIME SET MIN ************/
		else if (state == ST_WAIT_TIME_SET_MIN)
		{
			LCD_Colon(TRUE);	// turn on the ":" on LCD
			gArrow8=TRUE;		// turn on the arrow under der minutes-digits

			if (input == KEY_PLUS)
				gWaitMin++;
			if (input == KEY_MINUS)
				gWaitMin--;

			if (gWaitMin > 99)
				gWaitMin = 99;

			if (gWaitMin < 0)
				gWaitMin = 0;

			if ( (input == KEY_PREV) || (input == KEY_NEXT) )
			{
				state = ST_WAIT_TIME_SET_SEC;
				gArrow8=FALSE;		// turn off the arrow under der minutes-digits
				eeprom_write_byte_169(E2PROM_WAITMIN,gWaitMin);	// store the changed setting in EEPROM
			}

			if (input == KEY_ENTER)
			{
				state = ST_WAIT_TIME;
				LCD_Colon(FALSE);	// turn off the ":" on LCD
				gArrow8=FALSE;		// turn off the arrow under der seconds-digits
				eeprom_write_byte_169(E2PROM_WAITMIN,gWaitMin);	// store the changed setting in EEPROM
			}

			minH = CHAR2BCD2(gWaitMin);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			
			secH = CHAR2BCD2(gWaitSec);
			secL = (secH & 0x0F) + '0';
			secH = (secH >> 4) + '0';

            LCD_putc(0, 'W');
            LCD_putc(1, ' ');
            LCD_putc(2, minH);
            LCD_putc(3, minL);
            LCD_putc(4, secH);
            LCD_putc(5, secL);
            LCD_putc(6, '\0');
		}
/************* SHOOT COUNT ********************/
		else if (state == ST_SHOOT_COUNT)
		{
		    LCD_puts_f(PSTR("COUNT"),0);	// output string to LCD

			if (input == KEY_PLUS)
				state = ST_WAIT_TIME;
			if (input == KEY_MINUS)
				state = ST_SHOOT_TIME;
			if (input == KEY_PREV)
				state = ST_SET;

			if (input == KEY_NEXT)
				state = ST_SHOOT_COUNT_SET;
			
		}
/************* SHOOT COUNT ********************/
		else if (state == ST_SHOOT_COUNT_SET)
		{
			if (input == KEY_PLUS)
				gShootCount++;
			if (input == KEY_MINUS)
				gShootCount--;

			if (gShootCount > 99)
				gShootCount = 99;

			if (gShootCount < 0)
				gShootCount = 0;

			if (input == KEY_ENTER)
			{
				state = ST_SHOOT_COUNT;
				eeprom_write_byte_169(E2PROM_SHOOTCOUNT,gShootCount);	// store the changed setting in EEPROM
			}


			minH = CHAR2BCD2(gShootCount);
			minL = (minH & 0x0F) + '0';
			minH = (minH >> 4) + '0';

			
            LCD_putc(0, 'C');
            LCD_putc(1, ' ');
            LCD_putc(2, ' ');
            LCD_putc(3, ' ');
            LCD_putc(4, minH);
            LCD_putc(5, minL);
            LCD_putc(6, '\0');
		}

/************* OPTION BUZZER ******************/
		else if (state == ST_OPTION_BUZZER)
		{
		    LCD_puts_f(PSTR("BUZZER"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_BUZZER_SET;

			if (input == KEY_MINUS)
				state = ST_OPTION_CLICK;

			if (input == KEY_PLUS)
				state = ST_OPTION_TEMP;
		}
/************* OPTION BUZZER SET **************/
		else if (state == ST_OPTION_BUZZER_SET)
		{

			if (gBuzzer)
		    	LCD_puts_f(PSTR("ON"),0);
			else
		    	LCD_puts_f(PSTR("OFF"),0);

			if (input == KEY_PREV)
			{
				state = ST_OPTION_BUZZER;
				eeprom_write_byte_169(E2PROM_BUZZER,gBuzzer);	// store the buzzer setting in EEPROM
			}

			if ( (input == KEY_PLUS) || (input == KEY_MINUS) )
				gBuzzer ^= 1;	// toggle the buzzer setting
		}
/************* OPTION CLICK *******************/
		else if (state == ST_OPTION_CLICK)
		{
		    LCD_puts_f(PSTR("CLICK"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_CLICK_SET;

			if (input == KEY_MINUS)
				state = ST_OPTION_LED;

			if (input == KEY_PLUS)
				state = ST_OPTION_BUZZER;
		}
/************* OPTION CLICK SET ***************/
		else if (state == ST_OPTION_CLICK_SET)
		{

			if (gClick)
		    	LCD_puts_f(PSTR("ON"),0);
			else
		    	LCD_puts_f(PSTR("OFF"),0);

			if (input == KEY_PREV)
			{
				state = ST_OPTION_CLICK;
				KeyClickStatus= gClick;		// update the status
				eeprom_write_byte_169(E2PROM_CLICK,gClick);	// store the click setting in EEPROM
			}

			if ( (input == KEY_PLUS) || (input == KEY_MINUS) )
				gClick ^= 1;	// toggle the click setting
		}

/************* OPTION LED *********************/
		else if (state == ST_OPTION_LED)
		{
		    LCD_puts_f(PSTR("LED"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_LED_SET;

			if (input == KEY_MINUS)
				state = ST_OPTION_TEMP;

			if (input == KEY_PLUS)

				state = ST_OPTION_CLICK;
		}
/************* OPTION LED SET *****************/
		else if (state == ST_OPTION_LED_SET)
		{

			if (gLED == LED_STATE_OFF)
		    	LCD_puts_f(PSTR("OFF"),0);
			else if (gLED == LED_STATE_ON)
		    	LCD_puts_f(PSTR("ON"),0);
			else if (gLED == LED_STATE_LIGHT)
		    	LCD_puts_f(PSTR("LIGHT"),0);
			else if (gLED == LED_STATE_AUTO)
		    	LCD_puts_f(PSTR("AUTO"),0);

			if (input == KEY_PREV)
			{
				state = ST_OPTION_LED;
				eeprom_write_byte_169(E2PROM_LED,gLED);	// store the LED setting in EEPROM
			}

			if (input == KEY_MINUS)
			{
				gLED ++;
				if (gLED > LED_STATE_AUTO)
					gLED = LED_STATE_OFF;
			}
			if (input == KEY_PLUS)
			{
				gLED --;
				if (gLED < LED_STATE_OFF)
					gLED = LED_STATE_AUTO;
			}
		}
/************* OPTION TEMP ********************/
		else if (state == ST_OPTION_TEMP)
		{
		    LCD_puts_f(PSTR("TEMP"),0);	// output string to LCD

			if (input == KEY_PREV)
				state = ST_OPTION;

			if (input == KEY_NEXT)
				state = ST_OPTION_TEMP_SHOW;

			if (input == KEY_MINUS)
				state = ST_OPTION_BUZZER;

			if (input == KEY_PLUS)

				state = ST_OPTION_LED;
		}
/************* OPTION TEMP SHOW ***************/
		else if (state == ST_OPTION_TEMP_SHOW)
		{
			if (input == KEY_PREV)
				state = ST_OPTION_TEMP;
			ADMUX = TEMPERATURE_SENSOR;
			ADC_periphery();

		}


        LCD_UpdateRequired(TRUE, 0);   
	}
	
	
	return 0; // mt 
}







/*****************************************************************************
*
*   Function name : Initialization
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Initializate the different modules
*
*****************************************************************************/
void Initialization(void)
{
//    char tst;           // dummy

    OSCCAL_calibration();       // calibrate the OSCCAL byte
        
    CLKPR = (1<<CLKPCE);        // set Clock Prescaler Change Enable

    // set prescaler = 8, Inter RC 8Mhz / 8 = 1Mhz
    CLKPR = (1<<CLKPS1) | (1<<CLKPS0);

    // Disable Analog Comparator (power save)
    ACSR = (1<<ACD);

    // Disable Digital input on PF0-2 (power save)
    DIDR1 = (7<<ADC0D);

    // mt PORTB = (15<<PORTB0);       // Enable pullup on 
	PORTB = (15<<PB0);       // Enable pullup on 
    // mt PORTE = (15<<PORTE4);
	PORTE = (15<<PE4);

    sbiBF(DDRB, 5);               // set OC1A as output
    sbiBF(PORTB, 5);              // set OC1A high
            
    Button_Init();              // Initialize pin change interrupt on joystick
    
    RTC_init();                 // Start timer2 asynchronous, used for RTC clock

    Timer0_Init();              // Used when playing music etc.

    USART_Init(12);             // Baud rate = 9600bps
    
    //DF_SPI_init();              // init the SPI interface to communicate with the DataFlash
    
    //tst = Read_DF_status();

    //DF_CS_inactive;             // disable DataFlash
        
    LCD_Init();                 // initialize the LCD
}










/*****************************************************************************
*
*   Function name : Delay
*
*   Returns :       None
*
*   Parameters :    unsigned int millisec
*
*   Purpose :       Delay-loop
*
*****************************************************************************/
void Delay(unsigned int millisec)
{
    // mt, int i did not work in the simulator:  int i; 
	uint8_t i;
    
    while (millisec--)
        	//mt: for (i=0; i<125; i++);
			for (i=0; i<125; i++)  
				asm volatile ("nop"::);
}




/*****************************************************************************
*
*   Function name : OSCCAL_calibration
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Calibrate the internal OSCCAL byte, using the external 
*                   32,768 kHz crystal as reference
*
*****************************************************************************/
void OSCCAL_calibration(void)
{
    unsigned char calibrate = FALSE;
    int temp;
    unsigned char tempL;

    CLKPR = (1<<CLKPCE);        // set Clock Prescaler Change Enable
    // set prescaler = 8, Inter RC 8Mhz / 8 = 1Mhz
    CLKPR = (1<<CLKPS1) | (1<<CLKPS0);
    
    TIMSK2 = 0;             //disable OCIE2A and TOIE2

    ASSR = (1<<AS2);        //select asynchronous operation of timer2 (32,768kHz)
    
    OCR2A = 200;            // set timer2 compare value 

    TIMSK0 = 0;             // delete any interrupt sources
        
    TCCR1B = (1<<CS10);     // start timer1 with no prescaling
    TCCR2A = (1<<CS20);     // start timer2 with no prescaling

    while((ASSR & 0x01) | (ASSR & 0x04));       //wait for TCN2UB and TCR2UB to be cleared

    Delay(1000);    // wait for external crystal to stabilise
    
    while(!calibrate)
    {
        cli(); // mt __disable_interrupt();  // disable global interrupt
        
        TIFR1 = 0xFF;   // delete TIFR1 flags
        TIFR2 = 0xFF;   // delete TIFR2 flags
        
        TCNT1H = 0;     // clear timer1 counter
        TCNT1L = 0;
        TCNT2 = 0;      // clear timer2 counter
           
	// shc/mt while ( !(TIFR2 && (1<<OCF2A)) );   // wait for timer2 compareflag    
        while ( !(TIFR2 & (1<<OCF2A)) );   // wait for timer2 compareflag

        TCCR1B = 0; // stop timer1

        sei(); // __enable_interrupt();  // enable global interrupt
    
        // shc/mt if ( (TIFR1 && (1<<TOV1)) )
	if ( (TIFR1 & (1<<TOV1)) )
        {
            temp = 0xFFFF;      // if timer1 overflows, set the temp to 0xFFFF
        }
        else
        {   // read out the timer1 counter value
            tempL = TCNT1L;
            temp = TCNT1H;
            temp = (temp << 8);
            temp += tempL;
        }
    
        if (temp > 6250)
        {
            OSCCAL--;   // the internRC oscillator runs to fast, decrease the OSCCAL
        }
        else if (temp < 6120)
        {
            OSCCAL++;   // the internRC oscillator runs to slow, increase the OSCCAL
        }
        else
            calibrate = TRUE;   // the interRC is correct
    
        TCCR1B = (1<<CS10); // start timer1
    }
}
